#include "toglimg.h"
#include "jpeglib.h"

GLboolean toglCheckExtensionJPEG(char filename[])
{
	if (toglCheckExtension(filename, ".jpg")) {
		return GL_TRUE;
	}
	if (toglCheckExtension(filename, ".jpeg")) {
		return GL_TRUE;
	}
	if (toglCheckExtension(filename, ".jfif")) {
		return GL_TRUE;
	}

	return GL_FALSE;
}

GLboolean toglCheckImageJPEG(char file_name[])
{
	FILE			*fp;
    unsigned char	sig[3];


	sig[2] = '\0';

    if ((fp = fopen(file_name, "rb")) == NULL) {
		return GL_FALSE;
	}

	if (fread(sig, sizeof(unsigned char), 2, fp) != 2) {
		fclose(fp);
		return GL_FALSE;
    }
	
	fclose(fp);

#ifdef	_DEBUG	
	fprintf(stderr, "MAGIC   : %2.2x%2.2x\n", sig[0], sig[1]);
#endif	/* _DEBUG */

    if (sig[0] != 0xff || sig[1] != 0xd8) {
		return GL_FALSE;
    }

	return GL_TRUE;
}

GLboolean toglReadImageJPEG(char filename[], TOGLimage *img)
{
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
	JSAMPARRAY buff;

	FILE			*input_file;
	register int	y;
	int				row_stride;


	if (toglCheckImageJPEG(filename) == GL_FALSE) {
		return GL_FALSE;
	}

	if ((input_file = fopen(filename, "rb")) == NULL) {
		return GL_FALSE;
	}

	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);

	jpeg_stdio_src(&cinfo, input_file);

	(void) jpeg_read_header(&cinfo, TRUE);
	(void) jpeg_start_decompress(&cinfo);

	img->m_width = cinfo.output_width;
	img->m_height = cinfo.output_height;
	img->m_components = cinfo.output_components;

	if (!toglAllocImage(img)) {
		(void) jpeg_finish_decompress(&cinfo);
		jpeg_destroy_decompress(&cinfo);

		fclose(input_file);

		return GL_FALSE;
	}

	row_stride = cinfo.output_width * cinfo.output_components;
	buff = (*cinfo.mem->alloc_sarray)
		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

	/* Process data */
	y = 0;
	while (cinfo.output_scanline < cinfo.output_height) {
		register unsigned int i;
		unsigned char *p, *pbuff;

		jpeg_read_scanlines(&cinfo, buff, 1);

		p = img->m_buffer + (cinfo.output_height - y - 1) * cinfo.output_width * 4;
		for(i = 0; i < cinfo.output_width; i++) {
			pbuff = buff[0] + i * cinfo.output_components;

			switch (cinfo.output_components) {
				case 4:
					*(p++) = *(pbuff);
					*(p++) = *(pbuff + 1);
					*(p++) = *(pbuff + 2);
					*(p++) = *(pbuff + 3);
					break;
				case 3:
					*(p++) = *(pbuff);
					*(p++) = *(pbuff + 1);
					*(p++) = *(pbuff + 2);
					*(p++) = 255;
					break;
				case 2:
					*(p++) = *(pbuff);
					*(p++) = *(pbuff);
					*(p++) = *(pbuff);
					*(p++) = *(pbuff + 1);
					break;
				case 1:
				default:
					*(p++) = *(pbuff);
					*(p++) = *(pbuff);
					*(p++) = *(pbuff);
					*(p++) = 255;
					break;
			}

		}

		y++;
	}

	(void) jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);

	fclose(input_file);

	return GL_TRUE;
}

GLboolean toglWriteImageJPEG(char filename[], TOGLimage *img, int quality)
{
	unsigned char *buff;

	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;

	FILE		*outfile;		/* target file */
	JSAMPROW	row_pointer[1];	/* pointer to JSAMPLE row[s] */
	int			row_stride;		/* physical row width in image buffer */

	
	if ((buff = (unsigned char *)calloc(img->m_width * 3, sizeof(unsigned char))) == NULL) {
		return GL_FALSE;
	}

	if ((outfile = fopen(filename, "wb")) == NULL) {
		free(buff);
		return GL_FALSE;
	}

	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);

	jpeg_stdio_dest(&cinfo, outfile);

	cinfo.image_width = img->m_width; 	/* image width and height, in pixels */
	cinfo.image_height = img->m_height;

	switch (img->m_components) {
		case 3:
			cinfo.input_components = img->m_components;
			cinfo.in_color_space = JCS_RGB;
			break;
		case 1:
			cinfo.input_components = img->m_components;
			cinfo.in_color_space = JCS_GRAYSCALE;
			break;
		default:
			cinfo.input_components = 3;
			cinfo.in_color_space = JCS_RGB;
			break;
	}

	jpeg_set_defaults(&cinfo);

	jpeg_set_quality(&cinfo, quality, TRUE);

	jpeg_start_compress(&cinfo, TRUE);

//	row_stride = image_width * 3;	/* JSAMPLEs per row in image_buffer */
	row_stride = img->m_width * 4;	/* JSAMPLEs per row in image_buffer */

	while (cinfo.next_scanline < cinfo.image_height) {
		register int	x;
		unsigned char *p, *pbuff;
		
		p = buff;
		pbuff = img->m_buffer + (img->m_height - cinfo.next_scanline - 1) * row_stride;
		for(x = 0; x < img->m_width; x++) {
			switch (cinfo.input_components) {
				case 1:
					*(p++) = *(pbuff);
					pbuff += 4;
					break;
				default:
					*(p++) = *(pbuff++);
					*(p++) = *(pbuff++);
					*(p++) = *(pbuff++);
					pbuff++;
					break;
			}
		}

		row_pointer[0] = & buff[0];
		(void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
	}

	jpeg_finish_compress(&cinfo);

	fclose(outfile);

	jpeg_destroy_compress(&cinfo);

	free(buff);

	return GL_TRUE;
}
